//----------------------------- WAVE TEXT -------------------------------
//                by Carl Paulson      Cognitive Portals
//                 http://members.tripod.com/~SleepyCP/
//                          cpaulson@home.com
//-----------------------------------------------------------------------
//
//  Suggested HTML Input:
/*
  <APPLET code="Cylon.class" width=600 height=40>
      <param name=MSG        value="My Bonnie Lies Over the Ocean">

      <param name=Frequency  value="1");
      <param name=Amplitude  value="1");

      <param name=FONT       value="Helvetica");
      <param name=STYLE      value="0");
      <param name=SIZE       value="18");

      <param name=PauseIn    value="10">

      <param name=BG_Color   value="white">
      <param name=FG_Color   value="blue">


      <param name=DeltaH     value="0">   // adjust the Horizontal msg pos
      <param name=DeltaV     value="-20"> // adjust the Verticle msg pos

      <CENTER><FONT SIZE=3>{If you can read this, your browser dosen`t support Java.}</FONT><BR>
      <FONT FACE="Helvetica" SIZE=7>I Liked Battlestar Galactica</FONT></CENTER><BR>
  </APPLET>
*/
//-----------------------------------------------------------------------

import java.applet.Applet;
import java.util.*;
import java.awt.*;

public class Wave extends Applet implements Runnable {

    private boolean     Bool1    = false;
    private String      Message;
    private int         pause,      Frequency,  Amplitude;
    private FontMetrics fm;
    public  Thread      myThread;
    private Color       ColorBG,    ColorFG;
    private int         xLoc[],     yLoc[];
    private int         fontHeight, MsgLength,  x,   y;
    private Graphics    gBuffer;
    private Image       iBuffer;
    private int         Height1,    Width1;
    private int         iCount;
    private double      DeltaPhase;

    public void init() {

        Frequency  = getIntParam("FREQUENCY",1,1,3);
        Amplitude  = getIntParam("AMPLITUDE",1,0,2);

        setFont(getFontParam("FONT","Helvetica","STYLE",1,"SIZE",12));
        fm         = getFontMetrics(getFont());
        fontHeight = fm.getHeight();

        Message    = getStringParam("MSG","Default Message");
        MsgLength  = Message.length();

        iCount     = 0;
        xLoc       = new int[MsgLength + 1];
        yLoc       = new int[MsgLength + 1];
        DeltaPhase = Math.PI*((double)(Frequency))/((double)(MsgLength));
        yLoc[0]    = 0;
        xLoc[0]    = 0;
        for (int i=0; i < MsgLength; i++) {
            String S1 = Message.substring(0, i+1);
            xLoc[i+1] = fm.stringWidth(S1);
            double z  = (((double)(fontHeight*Amplitude))*Math.sin(((double)i)*DeltaPhase));
            yLoc[i+1] = (int)z;
            System.out.println("i, z, yLoc[i+1] = "+i+"  "+z+"  "+yLoc[i+1]);
        }

        ColorFG    = getColorParam("FG_Color", "RED");
        
        setForeground(ColorFG);
        ColorBG    = getColorParam("BG_Color", "white");
        setBackground(ColorBG);

        int DEL_H  = getIntParam("DeltaH",0,-100,100);
        Width1     = size().width;
        x          = DEL_H + (Width1 - xLoc[MsgLength - 1])/2;

        int DEL_V  = getIntParam("DeltaV",0,-100,100);
        Height1    = size().height;
        y          = (Height1 - fontHeight)/2 - DEL_V;

        pause      = getIntParam("PauseIn",100,0,500);
    }

    // ------------------------------------------------------------------

    public void start() {
        if (!Bool1) {
            if (myThread == null) 
                myThread = new Thread(this);
            myThread.start();
        }
    }

    //-------------------------------------------------------------------

    public void stop() {
        myThread = null;
        myThread.stop();
    }

    //-------------------------------------------------------------------
                            /** @deprecated */
    public boolean mouseDown(Event event, int i, int j) {
        if (Bool1) {
            Bool1 = false;
            start();
        } else {
            Bool1 = true;
            stop();
        }
        return true;
    }

    // --------------------- implement Runnable -------------------------

    public void run() {	

        while (myThread != null) {
            try {
                myThread.sleep (pause);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            repaint();
        }
    }

    // ---------------------  Create a buffer --=------------------------

    public void createBuffers() { 
        iBuffer = createImage(Width1, Height1);
        gBuffer = iBuffer.getGraphics();
    }

    // -------------- Override: Prevent Clearing Screen -----------------

    public void update(Graphics g) { 
        paint(g);
    }

    // ------------------------------------------------------------------

    public synchronized void paint(Graphics g) {

        int j, k, L;

        if(gBuffer == null) createBuffers();   // Create buffer
        gBuffer.setColor(ColorBG);
        gBuffer.fillRect(0, 0, Width1, Height1);

        int    x1, y1;
        String S1;

        for (int i=0; i < MsgLength; i++) {
            j  = i + iCount;
            L  = j - (j/MsgLength)*MsgLength; // Modulus
            S1 = Message.substring(i, i+1);
            x1 = xLoc[i] + x;
            y1 = yLoc[L] + y;
            gBuffer.setColor(ColorFG);
            gBuffer.drawString(S1, x1, y1);
        }

        g.drawImage(iBuffer, 0, 0, this); // Draw the buffer on the screen

        iCount++;
    }

    // ------------------------ getString -------------------------------

    public String getStringParam(String x, String def_X) {
        String str;
        str = getParameter(x);
        if (str == null) str = def_X;
        return str;
    }

    // ----------------------- getInteger -------------------------------

    public int getIntParam(String i, int def_I, int min_i, int max_i) {
        int i_in;
        try {
            i_in = Integer.parseInt(getParameter(i));
            if ((i_in < min_i)||(i_in > max_i)) throw new Exception();
        } catch (Exception e) {
            System.out.println(i+" invalid; using default value: "+def_I);
            i_in = def_I;
        }	
        return i_in;
    }

    // ------------------------ getColor --------------------------------

    public Color getColorParam(String c, String def_C) {

        Color Color1 = Color.lightGray;      // default color
        if (def_C != null) {
            if      (def_C.equalsIgnoreCase("black"))     Color1 = Color.black;
            else if (def_C.equalsIgnoreCase("blue"))      Color1 = Color.blue;
            else if (def_C.equalsIgnoreCase("cyan"))      Color1 = Color.cyan;
            else if (def_C.equalsIgnoreCase("darkGray"))  Color1 = Color.darkGray;
            else if (def_C.equalsIgnoreCase("gray"))      Color1 = Color.gray;
            else if (def_C.equalsIgnoreCase("green"))     Color1 = Color.green;
            else if (def_C.equalsIgnoreCase("lightGray")) Color1 = Color.lightGray;
            else if (def_C.equalsIgnoreCase("magenta"))   Color1 = Color.magenta;
            else if (def_C.equalsIgnoreCase("orange"))    Color1 = Color.orange;
            else if (def_C.equalsIgnoreCase("pink"))      Color1 = Color.pink;
            else if (def_C.equalsIgnoreCase("red"))       Color1 = Color.red;
            else if (def_C.equalsIgnoreCase("white"))     Color1 = Color.white;
            else if (def_C.equalsIgnoreCase("yellow"))    Color1 = Color.yellow;
        }

        StringTokenizer st;
        String rgbDelim = ":,.";
        String c_str = getParameter(c);
        if (c_str != null) st = new StringTokenizer(c, rgbDelim);
        else return Color1;

        if      (c_str.equalsIgnoreCase("black"))     return Color.black;
        else if (c_str.equalsIgnoreCase("blue"))      return Color.blue;
        else if (c_str.equalsIgnoreCase("cyan"))      return Color.cyan;
        else if (c_str.equalsIgnoreCase("darkGray"))  return Color.darkGray;
        else if (c_str.equalsIgnoreCase("gray"))      return Color.gray;
        else if (c_str.equalsIgnoreCase("green"))     return Color.green;
        else if (c_str.equalsIgnoreCase("lightGray")) return Color.lightGray;
        else if (c_str.equalsIgnoreCase("magenta"))   return Color.magenta;
        else if (c_str.equalsIgnoreCase("orange"))    return Color.orange;
        else if (c_str.equalsIgnoreCase("pink"))      return Color.pink;
        else if (c_str.equalsIgnoreCase("red"))       return Color.red;
        else if (c_str.equalsIgnoreCase("white"))     return Color.white;
        else if (c_str.equalsIgnoreCase("yellow"))    return Color.yellow;

        else if (st.countTokens() == 3) {
            Integer r = new Integer(st.nextToken());
            Integer g = new Integer(st.nextToken());
            Integer b = new Integer(st.nextToken());
            Color1    = new Color(r.intValue(), g.intValue(), b.intValue());
            return Color1;
            }
        return Color1;
    }

    // -------------------------- getFont  ------------------------------

    public Font getFontParam(String fnt, String def_Fnt, 
            String Style, int def_Style, String Size, int def_Size) {

        String font_Name  = getStringParam(fnt,   def_Fnt);
        int    font_Style = getIntParam   (Style, def_Style, 0, 3);
        int    font_Size  = getIntParam   (Size,  def_Size, 4, 100);

        return (new Font(font_Name, font_Style, font_Size));
    }

    // ------------------------------------------------------------------

}

